@@ -187,11 +187,12 @@ static ssize_t read_net_internal(int fd, void *buf, size_t count, int socketStat
 	return(retval);
 }
 
-static int open_net_http(char *user, char *pass, char *host, int port, char *path, int flags, off_t offset) {
+static int open_net_http(char *user, char *pass, char *host, int port, char *path, int flags, off_t offset, ssize_t *length) {
+	unsigned long content_length = 0;
+	ssize_t read_ret, sink_bytes = 0;
 	ssize_t send_ret, recv_ret;
 	char httpcmd[4096], charbuf, httpreply[1024], *http_response;
 	char garbage[4096];
-	ssize_t read_ret, sink_bytes = 0;
 	int bytes_to_read;
 	int fd;
 	int snprintf_ret;
@@ -258,6 +259,15 @@ static int open_net_http(char *user, char *pass, char *host, int port, char *pat
 							return(-1);
 					}
 				} else {
+					if (strncasecmp(httpreply, "Content-Length:", 15) == 0) {
+						content_length = strtoul(httpreply + 16, NULL, 10);
+						if (offset != 0 && http_response_code == 206) {
+							content_length += offset;
+						}
+						if (length) {
+							*length = content_length;
+						}
+					}
 					if (http_response_code == 301 || http_response_code == 302) {
 						if (strncasecmp(httpreply, "Location:", 9) == 0) {
 							net_close(fd);
@@ -285,12 +295,11 @@ static int open_net_http(char *user, char *pass, char *host, int port, char *pat
 	}
 
 	if (sink_bytes) {
-		while (offset) {
-			bytes_to_read = offset;
+		while (sink_bytes) {
+			bytes_to_read = sink_bytes;
 			if (bytes_to_read > sizeof(garbage)) {
 				bytes_to_read = sizeof(garbage);
 			}
-			SPOTVAR_LU(bytes_to_read);
 
 			read_ret = read_net_internal(fd, garbage, bytes_to_read, 1);
 			if (read_ret <= 0) {
@@ -299,18 +308,18 @@ static int open_net_http(char *user, char *pass, char *host, int port, char *pat
 				return(-1);
 			}
 
-			offset -= read_ret;
+			sink_bytes -= read_ret;
 		}
 	}
 
 	return(fd);
 }
 
-static int open_net_ftp(char *user, char *pass, char *host, int port, char *path, int flags, off_t offset) {
+static int open_net_ftp(char *user, char *pass, char *host, int port, char *path, int flags, off_t offset, ssize_t *length) {
 	return(-1);
 }
 
-static int open_net_internal(const char *pathname, int flags, mode_t mode, int *socket, off_t offset) {
+static int open_net_internal(const char *pathname, int flags, mode_t mode, int *socket, off_t offset, ssize_t *length) {
 	char *pathname_dup;
 	char *scheme, *user, *pass, *host, *path;
 	off_t lseek_ret;
@@ -352,15 +361,28 @@ static int open_net_internal(const char *pathname, int flags, mode_t mode, int *
 
 	if (parse_ret < 0 || port == 0 || !host || !scheme) {
 		free(pathname_dup);
-		return(open(pathname, flags, mode));
+		retval = open(pathname, flags, mode);
+		if (offset && retval >= 0) {
+			lseek_ret = lseek(retval, offset, SEEK_SET);
+			if (lseek_ret != offset) {
+				close(retval);
+				return(-1);
+			}
+		}
+		return(retval);
+
+	}
+
+	if (length) {
+		*length = -1;
 	}
 
 	if (strcasecmp(scheme, "http") == 0) {
-		retval = open_net_http(user, pass, host, port, path, flags, offset);
+		retval = open_net_http(user, pass, host, port, path, flags, offset, length);
 	}
 
 	if (strcasecmp(scheme, "ftp") == 0) {
-		retval = open_net_ftp(user, pass, host, port, path, flags, offset);
+		retval = open_net_ftp(user, pass, host, port, path, flags, offset, length);
 	}
 
 	free(pathname_dup);
@@ -384,8 +406,9 @@ static int open_net_internal(const char *pathname, int flags, mode_t mode, int *
 }
 
 NETFILE *fopen_net(const char *pathname, const char *mode) {
-	int is_sock;
 	NETFILE *ret;
+	int is_sock;
+	ssize_t length;
 
 	ret = malloc(sizeof(*ret));
 
@@ -393,7 +416,7 @@ NETFILE *fopen_net(const char *pathname, const char *mode) {
 		return(NULL);
 	}
 
-	ret->fd = open_net_internal(pathname, O_RDONLY, 0666, &is_sock, 0);
+	ret->fd = open_net_internal(pathname, O_RDONLY, 0666, &is_sock, 0, &length);
 
 	if (ret->fd < 0) {
 		free(ret);
@@ -404,9 +427,9 @@ NETFILE *fopen_net(const char *pathname, const char *mode) {
 	ret->buf_s = ret->buf = malloc(ret->bufsize);
 	ret->bufused = 0;
 	ret->eof = 0;
-	ret->socket = is_sock;
 	ret->pos = 0;
-	ret->length = -1;
+	ret->socket = is_sock;
+	ret->length = length;
 	ret->url = strdup(pathname);
 
 	return(ret);
@@ -417,6 +440,10 @@ int feof_net(NETFILE *stream) {
 		return(-1);
 	}
 
+	if (stream->fd < 0) {
+		return(-1);
+	}
+
 	return(stream->eof);
 }
 
@@ -587,7 +614,7 @@ int fclose_net(NETFILE *stream) {
 int open_net(const char *pathname, int flags, mode_t mode) {
 	int tmp;
 
-	return(open_net_internal(pathname, flags, mode, &tmp, 0));
+	return(open_net_internal(pathname, flags, mode, &tmp, 0, NULL));
 }
 
 ssize_t read_net(int fd, void *buf, size_t count) {
@@ -600,16 +627,24 @@ off_t lseek_net(int filedes, off_t offset, int whence) {
 
 int fseek_net(NETFILE *stream, long offset, int whence) {
 	static char garbage[4096];
+	ssize_t newlength;
 	size_t fread_ret, bytes_to_read;
 	off_t newoffset, lseek_ret;
 	int newfd;
 
+	if (!stream) {
+		return(-1);
+	}
+	if (stream->fd < 0) {
+		return(-1);
+	}
+
 	switch (whence) {
 		case SEEK_CUR:
 			newoffset = stream->pos + offset;
 			break;
 		case SEEK_END:
-			newoffset = stream->length - offset;
+			newoffset = stream->length + offset;
 			break;
 		case SEEK_SET:
 			newoffset = offset;
@@ -620,6 +655,10 @@ int fseek_net(NETFILE *stream, long offset, int whence) {
 			break;
 	}
 
+	if (newoffset < 0) {
+		return(-1);
+	}
+
 	lseek_ret = lseek(stream->fd, newoffset, SEEK_SET);
 	if (lseek_ret != ((off_t) -1)) {
 		stream->pos = lseek_ret;
@@ -642,7 +681,7 @@ int fseek_net(NETFILE *stream, long offset, int whence) {
 		}
 	}
 
-	newfd = open_net_internal(stream->url, O_RDONLY, 0666, NULL, newoffset);
+	newfd = open_net_internal(stream->url, O_RDONLY, 0666, NULL, newoffset, &newlength);
 
 	if (newfd < 0) {
 		return(-1);
@@ -654,6 +693,7 @@ int fseek_net(NETFILE *stream, long offset, int whence) {
 	stream->buf = stream->buf_s;
 	stream->bufused = 0;
 	stream->bufsize = stream->bufsize_s;
+	stream->eof = 0;
 
 	return(0);
 }
@@ -668,3 +708,14 @@ long ftell_net(NETFILE *stream) {
 
 	return(stream->pos);
 }
+
+long flength_net(NETFILE *stream) {
+	if (!stream) {
+		return(-1);
+	}
+	if (stream->fd < 0) {
+		return(-1);
+	}
+
+	return(stream->length);
+}
