@@ -39,7 +39,7 @@ typedef struct {
 	int eof;
 	int socket;
 	unsigned long pos;
-	unsigned long length;
+	long length;
 	char *url;
 } NETFILE;
 
@@ -74,6 +74,9 @@ long ftell_net(NETFILE *stream);
 /* ANSI fclose() clone */
 int fclose_net(NETFILE *stream);
 
+/* Return the length of a stream, if possible. -1 if not available */
+long flength_net(NETFILE *stream);
+
 #ifdef  __cplusplus
 }
 #endif
